//
// This source-code provides access to the 16bit
// version of cards.dll.
//	
// Each function in cards.dll is accessed using two
// wrapper functions. The outer wrapper provides
// the 32bit gateway (our application interface). This interface
// sets up the global variables for the inner wrapper 
// (the thunking wrapper) to use. The thunking function
// calls the 16bit DLL functions directly using an assembly hack.
//
// Left as an excercise for the reader: cdtAnimate(..)
//
//	Note: To call the LoadLibrary16 and FreeLibrary16 functions,
//        we have created an import library (cards16.lib) which imports
//        these functions by ordinal. See cards16.def for details.
//		  Don't forget to link to wow32.lib and cards16.lib!!!
//
#include <windows.h>
#include "cards.h"

//
//	The interface to this module
//
BOOL InitCardsDll16(void);
int  DeInitCardsDll16(void);

//
//	Function prototypes for the card functions in cards.dll 
//	(16bit version)
//	
//	You must define these function pointers in your main app.
//
extern pfcdtInit		cdtInit;
extern pfcdtDraw		cdtDraw;
extern pfcdtDrawEx		cdtDrawEx;
extern pfcdtAnimate		cdtAnimate;
extern pfcdtTerm		cdtTerm;


//
//
//	Undocumented functions from Kernel32.dll
//

#ifdef __cplusplus
extern "C" {
#endif

#include <wownt32.h>

HINSTANCE	WINAPI LoadLibrary16( PSTR );
void		WINAPI FreeLibrary16( HINSTANCE );
FARPROC		WINAPI GetProcAddress16( HINSTANCE, PSTR );

#ifdef __cplusplus
}
#endif

static HINSTANCE	hLib16;
static FARPROC		QT_Thunk;

//
// don't need prototypes, these are being called from assembler
//
static FARPROC cdtInit16;
static FARPROC cdtDraw16;
static FARPROC cdtTerm16;
static FARPROC cdtDrawEx16;

static WORD  h16bitmem;
static DWORD p16bitmem;

static int * p32bitmem;

static WORD  retval;


//
// we need to use globals for the thunking functions
// to work correctly. (i.e. we can't pass parameters
// to a thunking function)
//
static WORD      hdc16bit;
static short int global_x, global_y;
static short int global_card, global_type;
static DWORD	 global_cardback;
static short int global_width, global_height;
static int		 global_cardwidth, global_cardheight;

#pragma optimize( "", off )

static int CallcdtInitThunk()
{
	//put a dummy "scratch-pad" on the stack for QT_Thunk
	//need to make sure that the optimiser does not optimize
	//this away, so write to the first byte, and also
	//return a value from the array (see last statement of this function)
	unsigned char dummy[0x40];
	dummy[0] = 0;
		
	// allocate some memory in 16bit space!!!!!
	// This is necessary because cdtInit expects to be passed
	// the address of two integers. The 16bit function obviously
	// needs to be able to store values through these pointers, so
	// we need to allocate some memory in 16bit space.
	p16bitmem = WOWGlobalAllocLock16(GMEM_FIXED |GMEM_ZEROINIT| GMEM_SHARE, 32, &h16bitmem);

	//find the 32bit address
	p32bitmem = (int *)WOWGetVDMPointerFix(p16bitmem, 32, 1);

	//call cdtInit in 16bit code!!!
	__asm 
	{        
		mov		eax, [p16bitmem]
		push    eax
		add		ax, 4
		push    eax
		mov     edx, [cdtInit16]        
		call    QT_Thunk
		mov     [retval], ax    
	}
	
	// retrieve the width and height from the 16bit address space.
	global_cardwidth  = p32bitmem[0];
	global_cardheight = p32bitmem[1];

	//free the 16bit memory
	WOWGlobalUnlockFree16(p16bitmem);
	
	return retval;
}

#pragma optimize( "", on )

//
//  provide wrappers for the 16bit calls
//	these functions are called from the main 32bit app
//
static BOOL WINAPI cdtInitProxy(int *dx, int *dy)
{
	CallcdtInitThunk();
	*dx = global_cardwidth;
	*dy = global_cardheight;

	return TRUE;
}

#pragma optimize( "", off )

static WORD CallcdtDrawThunk()
{
	unsigned char dummy[0x40];
	dummy[0] = 0;

	__asm 
	{        
		xor		eax, eax
		mov		ax, [hdc16bit]
		push    ax			//hdc		(16bits)

		mov		ax, [global_x]
		push    ax			//x			(16bits)
		
		mov		ax, [global_y]
		push    ax			//y			(16bits)
		
		mov		ax, [global_card]
		push	ax			//cardnum	(16bits)
		
		mov		ax, [global_type]
		push	ax			//type		(16bits)
		
		mov		eax, [global_cardback]
		push	eax			//back		(32bits)
		
		mov     edx, [cdtDraw16]        
		call    QT_Thunk
		mov     [retval], ax    
	}

	return retval;
}

#pragma optimize( "", on )
static BOOL WINAPI cdtDrawProxy(HDC hdc, int x, int y, int card, int type, DWORD back)
{
	hdc16bit = WOWHandle16(hdc, WOW_TYPE_HDC);

	global_x =		(short)x;
	global_y =		(short)y;
	global_card =	(short)card;
	global_type =	(short)type;
	global_cardback=(short)back;

	return CallcdtDrawThunk();
//	return TRUE;
}

#pragma optimize( "", off )

static WORD CallcdtDrawExThunk()
{
	unsigned char dummy[0x40];
	dummy[0] = 0;

	__asm 
	{        
		xor		eax, eax
		mov		ax, [hdc16bit]
		push    ax			//hdc		(16bits)

		mov		ax, [global_x]
		push    ax			//x			(16bits)
		
		mov		ax, [global_y]
		push    ax			//y			(16bits)
		
		mov		ax, [global_width]
		push    ax			//x			(16bits)
		
		mov		ax, [global_height]
		push    ax			//y			(16bits)

		mov		ax, [global_card]
		push	ax			//cardnum	(16bits)
		
		mov		ax, [global_type]
		push	ax			//type		(16bits)
		
		mov		eax, [global_cardback]
		push	eax			//back		(32bits)
		
		mov     edx, [cdtDrawEx16]
		call    QT_Thunk
		mov     [retval], ax    
	}

	return retval;
}

#pragma optimize( "", on )

static BOOL WINAPI cdtDrawExProxy(HDC hdc, int x, int y, int width, int height, int card, int type, DWORD back)
{
	hdc16bit = WOWHandle16(hdc, WOW_TYPE_HDC);

	global_x =		(short)x;
	global_y =		(short)y;
	global_width =	(short)width;
	global_height = (short)height;
	global_card =	(short)card;
	global_type =	(short)type;
	global_cardback=(short)back;

	return CallcdtDrawExThunk();
}

//
//	Need to implement the CallcdtAnimateThunk() function..
//
static BOOL WINAPI cdtAnimateProxy(HDC hdc, int cardback, int x, int y, int frame)
{
	hdc16bit = WOWHandle16(hdc, WOW_TYPE_HDC);

	return FALSE;
}

#pragma optimize( "", off )

static WORD CallcdtTermThunk(void)
{
	unsigned char dummy[0x40];
	dummy[0] = 0;

	__asm 
	{
		mov     edx, [cdtTerm16]
		call    QT_Thunk
		mov     [retval], ax    
	}
	
	return dummy[1];
}

#pragma optimize( "", on )

static void WINAPI cdtTermProxy(void)
{
	CallcdtTermThunk();	
}


//
// This module uses LoadLibrary16 and FreeLibrary16
// from kernel32.dll.
//
BOOL InitCardsDll16(void)
{
	HMODULE hKernel;		//Usual Windows API calls
	hKernel  = GetModuleHandle("KERNEL32");
	QT_Thunk = GetProcAddress(hKernel, "QT_Thunk");

	hLib16 = LoadLibrary16("cards.dll");

	if(hLib16 == 0)
	{
		return FALSE;
	}

	if((UINT)hLib16 > 32)
	{
		cdtInit16 = (FARPROC)GetProcAddress16(hLib16, "cdtInit");
		cdtDraw16 = (FARPROC)GetProcAddress16(hLib16, "cdtDraw");
		cdtTerm16 = (FARPROC)GetProcAddress16(hLib16, "cdtTerm");

		cdtInit    = cdtInitProxy;
		cdtDraw    = cdtDrawProxy;
		cdtDrawEx  = cdtDrawExProxy;
		cdtAnimate = cdtAnimateProxy;
		cdtTerm    = cdtTermProxy;
	}
	
	return TRUE;
}

int DeInitCardsDll16(void)
{
	FreeLibrary16(hLib16);
	return TRUE;
}

