#include <windows.h>
#include <stdlib.h>

#include "cards.h"

char szAppName[] = "cards";

BOOL InitCardsDll32(void);
int  DeInitCardsDll32(void);
BOOL InitCardsDll16(void);
int  DeInitCardsDll16(void);


//
//	Define function pointers to the card functions.
//  When we run on NT or 2000, these functions will point
//  directly into cards.dll. Under 95/98/ME, these functions
//  will point to the "Proxy" functions in cards16.c, which
//  do the thunking work for us.
//
pfcdtInit		cdtInit;
pfcdtDraw		cdtDraw;
pfcdtDrawEx		cdtDrawEx;
pfcdtAnimate	cdtAnimate;
pfcdtTerm		cdtTerm;

int width, height;

//
// Decide which version of the InitCardsDll 
// function to call. The function we end up
// calling will set the function pointers to
// to point to the correct target function.
//
BOOL InitCardsDll()
{
	if(GetVersion() < 0x80000000)	//Windows NT, 2000
	{
		return InitCardsDll32();
	}
	else							//Windows 95, 98, ME
	{
		return InitCardsDll16();
	}
}

BOOL DeInitCardsDll()
{
	if(GetVersion() < 0x80000000)	//Windows NT, 2000
	{
		return DeInitCardsDll32();
	}
	else							//Windows 95, 98, ME
	{
		return DeInitCardsDll16();
	}
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	int x, y, card, type;
	RECT rect;

	switch(msg)
	{
	case WM_CREATE:
		
		//
		//	Initialize cards.dll for this application
		//
		cdtInit(&width, &height);

		//
		//	Create a timer which will draw a new card each time it goes off
		//
		SetTimer(hwnd, 1, 30, 0);
		return 0;

	case WM_TIMER:
		
		// find out how big the window is
		GetClientRect(hwnd, &rect);

		hdc = GetDC(hwnd);

		x = rand() % (rect.right - width);
		y = rand() % (rect.bottom - height);
		card = MAKE_CARD_VALUE(rand() % 4, rand() % 13);
		type = ectFACES;

		//
		//	Draw a card!!
		//
		cdtDraw(hdc, x, y, card, type, 0);
		
		ReleaseDC(hwnd, hdc);
		return 0;

	case WM_DESTROY:
		
		//
		//	Remember to close down cards.dll properly
		//
		cdtTerm();

		PostQuitMessage(0);
		return 0;

	case WM_CLOSE:
		DestroyWindow(hwnd);
		return 0;
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmdLine, int nShowCmd)
{
	HWND		hwnd;
	MSG			msg;
	WNDCLASSEX	wndclass;

	if(!InitCardsDll())
	{
		MessageBox(0, "Failed to load cards.dll", szAppName, 0);
		return 0;
	}

	//Window class for the main application parent window
	wndclass.cbSize			= sizeof(wndclass);
	wndclass.style			= 0;
	wndclass.lpfnWndProc	= WndProc;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInst;
	wndclass.hIcon			= LoadIcon(0, MAKEINTRESOURCE(IDI_APPLICATION));
	wndclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground	= (HBRUSH)(COLOR_APPWORKSPACE+1);
	wndclass.lpszMenuName	= 0;
	wndclass.lpszClassName	= szAppName;
	wndclass.hIconSm		= LoadIcon (NULL, IDI_APPLICATION);

	RegisterClassEx(&wndclass);

	hwnd = CreateWindowEx(WS_EX_CLIENTEDGE, 
				szAppName,			// window class name
				"cards.dll demo",	// window caption
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT,			// initial x position
				CW_USEDEFAULT,			// initial y position
				CW_USEDEFAULT,			// initial x size
				CW_USEDEFAULT,			// initial y size
				NULL,					// parent window handle
				NULL,					// use window class menu
				hInst,					// program instance handle
				NULL);					// creation parameters

	ShowWindow(hwnd, nShowCmd);

	while(GetMessage(&msg, NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	DeInitCardsDll();

	return 0;
}